/**********************************************************************
 * This file is Copyright 1999-2000 by Alexander E. Mohr.             *
 * In order to use any part of this file, you must comply with the    *
 * license in "LICENSE."                                              *
 *********************************************************************/

#include "headers.h"

int utilities_to_code_strength( ASSIGNMENT * assignment,
				 UC_CURVE * uc_curve,
				 CONFIG_DATA * config )
{
  int total_symbols_used = 0;
  int source_symbols_used = 0;

  double current_util;
  int max_symbols = config->number_symbols;
  int N = config->number_descriptions;
  int i;
  int count;

#ifdef DEBUGGUIDE
  for( i = 0; i <= N; i++ )
    {
      printf("Rectify: Guide Utility: %4d %7.4f\n", i, assignment->guide_utility[i] );
    }
#endif

  current_util = uc_curve->actual_head->utility;
  assignment->actual_utility[0] = current_util;

  count = 0;
  for( i = 1; i <= N; i++ )
    {
      if ( current_util >= assignment->guide_utility[i]
	   || (!config->matching && total_symbols_used >= max_symbols) )
	{
#ifdef DEBUGGUIDE
	  printf("Assigning %3d: %f -> %f\n",
		 i, current_util, assignment->guide_utility[i] );
#endif
	  assignment->actual_utility[i] = current_util;
	  continue;
	}
      total_symbols_used += N;
      source_symbols_used += i;
      if ( !config->matching )
	assignment->code_strength[count] = i;
      count++;
      current_util = get_utility( source_symbols_used * config->symbol_size,
				  uc_curve->actual_head,
				  config );
      i--;
    }
  assignment->code_set = TRUE;
  assignment->actual_set = TRUE;
  printf("Used %d symbols in each description (%d source, %d total).\n",
	 count, source_symbols_used, total_symbols_used);
  return count;
}

double get_utility( int bits, UC_ENTRY *uc_entry, CONFIG_DATA * config )
{
  if ( uc_entry->cost == bits || uc_entry->next == NULL )
    return uc_entry->utility;

  if ( uc_entry->cost < bits )
    {
      double fraction;
      if ( uc_entry->next->cost <= bits )
	return get_utility( bits, uc_entry->next, config );

      fraction = (double) (bits - uc_entry->cost)
	/ (double) (uc_entry->next->cost - uc_entry->cost);
      
      return fraction * uc_entry->next->utility
	+ (1.0-fraction) * uc_entry->utility;
    }

  fprintf( stderr, "Error: rectify: get_utility().  Cost was > bits.\n" );
  exit(1);
}
